//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                  InsideBar.mq5 |
//|                                                                                                                                                                            hoz |
//|                                                                                                                                                                                |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
#property copyright "hoz"
#property link      ""
#property version   "1.00"
#property indicator_chart_window
#property indicator_buffers 4
#property indicator_plots 2

#property indicator_type1 DRAW_COLOR_ARROW
#property indicator_color1  clrRed, clrTurquoise, clrLightSkyBlue, clrGoldenrod
#property indicator_width1  2

#property indicator_type2 DRAW_COLOR_ARROW
#property indicator_color2  clrIndigo, clrAzure, clrSienna
#property indicator_width2  2
// ====================================================================== Включения и импорт внешних модулей ======================================================================+
#import "EnvironmentInfo\TimeSeries.ex5"
  int barsTotal(ENUM_TIMEFRAMES tf = PERIOD_CURRENT);
  int barShift(datetime initialBarTime, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);
  double barHighPrice(int shift, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);
  double barLowPrice(int shift, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);
  double barOpenPrice(int shift, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);
  double barClosePrice(int shift, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);
  datetime barOpenTime(int shift, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);
#import

#include <HOZ_Code\Enums\Location.mqh>
#include <HOZ_Code\Structures\DateTimeInstance.mqh>
#include <HOZ_Code\Classes\GraphicalPlotting\Draw.mqh>
#include <HOZ_Code\Classes\Environment\SymbolInstance.mqh>

// ===============================================================================  Input переменные  =============================================================================+
input bool    showInsideBarWithBearishBody = true;    // Искать паттерн 'Внутренний бар с медвежьим телом'?
input bool    showInsideBarWithBullishBody = true;    // Искать паттерн 'Внутренний бар с бычьим телом'?
input bool    checkBarBodySize = false;               // Проверять размер тела бара?
input double  insideBarMinBodySize = 1.0;             // Минимальный размер тела паттерна 'Внутренний бар'
input double  insideBarMaxBodySize = INT_MAX;         // Максимальный размер тела паттерна 'Внутренний бар'
input bool    checkBarShadowSize = false;             // Проверять размер тени бара?
input double  insideBarLowerShadowSize = 150.0;       // Относительный размер нижней тени паттерна 'Внутренний бар'
input double  insideBarUpperShadowSize = 150.0;       // Абсолютный размер верхней тени паттерна 'Внутренний бар'

input int     arrowShift = 15;
input bool    toDrawPatternsInfo = true;
input int     barsToCalculate = 3321;

SymbolInstance *ptrSym;

double insideBarWithBearishBody_buffer[];    // Буфер паттерна "Внутренний бар" с медвежьим телом бара
double insideBarWithBullishBody_buffer[];    // Буфер паттерна "Внутренний бар" с бычьим телом бара
double upperArrowColor[];                    // Буфер цвета верхней стрелочки
double lowerArrowColor[];                    // Буфер цвета нижней стрелочки

int    firstBarIndexToCalculate;

#define PREFIX "PA_PATTERN"                  // Префикс имён графических объектов

#define INSIDE_BAR_WITH_BEARISH_BODY   1
#define INSIDE_BAR_WITH_BULLISH_BODY   2

#define NO_DIRECTION           0
#define DOWNWARD_DIRECTION     1
#define UPWARDS_DIRECTION      2
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator initialization function                                                                                                                                       |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int OnInit() {
  ptrSym = new SymbolInstance(_Symbol);

  IndicatorSetString(INDICATOR_SHORTNAME, "Inside bar");
  IndicatorSetInteger(INDICATOR_DIGITS, _Digits);

  // === Связывание буферов с индексами, определение стилей ==========================
  SetIndexBuffer(0, insideBarWithBearishBody_buffer, INDICATOR_DATA);    // 1-ый буфер - "Молот" с медвежьим телом бара
  SetIndexBuffer(1, upperArrowColor, INDICATOR_COLOR_INDEX);             // 2-ой буфер - Цвет верхней стрелочки

  SetIndexBuffer(2, insideBarWithBullishBody_buffer, INDICATOR_DATA);    // 3-ой буфер - "Молот" с бычьим телом бара
  SetIndexBuffer(3, lowerArrowColor, INDICATOR_COLOR_INDEX);             // 4-ой буфер - Цвет нижней стрелочки

//  PlotIndexSetInteger(0, PLOT_DRAW_TYPE, DRAW_COLOR_ARROW);
  PlotIndexSetInteger(0, PLOT_ARROW, 218);
  PlotIndexSetInteger(0, PLOT_ARROW_SHIFT, 50);
  PlotIndexSetDouble(0, PLOT_EMPTY_VALUE, EMPTY_VALUE);

//  PlotIndexSetInteger(1, PLOT_DRAW_TYPE, DRAW_COLOR_ARROW);
  PlotIndexSetInteger(1, PLOT_ARROW, 217);
  PlotIndexSetInteger(1, PLOT_ARROW_SHIFT, -55);
  PlotIndexSetDouble(1, PLOT_EMPTY_VALUE, EMPTY_VALUE);

  ArraySetAsSeries(insideBarWithBearishBody_buffer, true);
  ArraySetAsSeries(upperArrowColor, true);
  ArraySetAsSeries(insideBarWithBullishBody_buffer, true);
  ArraySetAsSeries(lowerArrowColor, true);
//---
   return INIT_SUCCEEDED;
}
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator initialization function                                                                                                                                       |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void OnDeinit(const int reason) {
  delete ptrSym;
  deleteAllObjects();
}
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Удаление всех объектов, созданных индикатором                                                                                                                                  |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void deleteAllObjects() {
  ObjectsDeleteAll(0, PREFIX, 0, -1);
}
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Отрисовка указателей на найдённые паттерны в виде стрелочек, линии хай или лоу сформированных паттернов (в зависимости от типа паттерна) и подпись этих паттернов текстом      |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void drawPatternInfo(string patternName, int index, ENUM_LOCATION objLocation, uchar arrowCode, color clr) {
    string arrowName = Draw :: wingdings(PREFIX, patternName, objLocation, arrowCode, 2, clr, index, _Period);
    double arrowPrice = ObjectGetDouble(0, arrowName, OBJPROP_PRICE);
    double price1, price2;

    if (objLocation == LOCATION_BELOW) {
      price1 = price2 = barLowPrice(index);
//      Draw :: text(PREFIX, patternName, LOCATION_BELOW, arrowPrice, clrBlack, index, _Period);
    } else {
      price1 = price2 = barHighPrice(index);
//      Draw :: text(PREFIX, patternName, LOCATION_ABOVE, arrowPrice, clrBlack, index, _Period);
    }
    Draw :: line(PREFIX, patternName, barOpenTime(index), price1, barOpenTime(index - 4), price2, STYLE_SOLID, 2, clrBlack, index);
}
//==================================================================================================================================================================================
// setter значений индикаторных буферов. ===========================================================================================================================================
void setBuffers(int patternType,    // Тип паттерна
                int i) {            // Индекс обрабатываемого бара
  if (patternType == INSIDE_BAR_WITH_BEARISH_BODY) {
    insideBarWithBearishBody_buffer[i] = barHighPrice(i);
    insideBarWithBullishBody_buffer[i] = barLowPrice(i);
    if (showInsideBarWithBearishBody)
      drawPatternInfo("Inside bar with bearish body", i, LOCATION_ABOVE, 242, clrRed);
    return;

//    insideBarWithBearishBody_buffer[i] = EMPTY_VALUE;
  } else {
    insideBarWithBearishBody_buffer[i] = barHighPrice(i);
    insideBarWithBullishBody_buffer[i] = barLowPrice(i);
    if (showInsideBarWithBullishBody)
      drawPatternInfo("Inside bar with bullish body", i, LOCATION_BELOW, 241, clrBlue);
    return;

//    insideBarWithBullishBody_buffer[i] = EMPTY_VALUE;
  }

/*  if (toDrawPatternsInfo) {
    upperArrowColor[i] = 0;
    lowerArrowColor[i] = 2;
  } else {
    upperArrowColor[i] = EMPTY_VALUE;
    lowerArrowColor[i] = EMPTY_VALUE;
  }*/
}
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Поиск паттерна Hammer with bearish body                                                                                                                                        |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void findInsideBarWithBearishBody(int i) {
  double bodySize = barOpenPrice(i) - barClosePrice(i);
  double lowerShadowSize = barClosePrice(i) - barLowPrice(i);
  double upperShadowSize = barHighPrice(i) - barOpenPrice(i);
  
  if (barHighPrice(i) > barHighPrice(i + 1) || barLowPrice(i) < barLowPrice(i + 1)) return;
  if (bodySize < insideBarMinBodySize * ptrSym.pt() || bodySize > insideBarMaxBodySize * ptrSym.pt()) return;
  if (upperShadowSize > insideBarUpperShadowSize * ptrSym.pt() || lowerShadowSize > insideBarLowerShadowSize * ptrSym.pt()) return;
  
  setBuffers(INSIDE_BAR_WITH_BEARISH_BODY, i);
}
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Поиск паттерна Hammer with bullish body                                                                                                                                        |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void findInsideBarWithBullishBody(int i) {
  double bodySize = barClosePrice(i) - barOpenPrice(i);
  double lowerShadowSize = barOpenPrice(i) - barLowPrice(i);
  double upperShadowSize = barHighPrice(i) - barClosePrice(i);
  
  if (barHighPrice(i) > barHighPrice(i + 1) || barLowPrice(i) < barLowPrice(i + 1)) return;
  if (bodySize < insideBarMinBodySize * ptrSym.pt() || bodySize > insideBarMaxBodySize * ptrSym.pt()) return;
  if (upperShadowSize > insideBarUpperShadowSize * ptrSym.pt() || lowerShadowSize > insideBarLowerShadowSize * ptrSym.pt()) return;
  
  setBuffers(INSIDE_BAR_WITH_BULLISH_BODY, i);
}
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Расчет значений индикатора                                                                                                                                                     |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void checkingPatterns(int limit) {
  for (int i = limit; i > 0; i--) {
    findInsideBarWithBearishBody(i);
    findInsideBarWithBullishBody(i);
  }
}
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Определение индекса бара, с которого необходимо производить перерасчет                                                                                                         |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int recalcIndex(int calculatedYet) {
  firstBarIndexToCalculate = barsTotal() - 1;

  if (calculatedYet == 0) {
    ArrayInitialize(insideBarWithBearishBody_buffer, EMPTY_VALUE);
    ArrayInitialize(upperArrowColor, EMPTY_VALUE);
    ArrayInitialize(insideBarWithBullishBody_buffer, EMPTY_VALUE);
    ArrayInitialize(lowerArrowColor, EMPTY_VALUE);
    if (barsToCalculate > 0 && barsToCalculate < firstBarIndexToCalculate) {    // Если не нужно рассчитывать всю..
      firstBarIndexToCalculate = barsToCalculate;                               // ..историю, то начнем с указанного..
    }
    return firstBarIndexToCalculate;
  }
  return 2;

/*  if (calculatedYet == 0) {
    ArrayInitialize(insideBarWithBearishBody_buffer, EMPTY_VALUE);
    ArrayInitialize(insideBarWithBullishBody_buffer, EMPTY_VALUE);
  }*/
}
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                        Custom indicator iteration function                                                                     |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[]) {
//---
  if (ChartGetDouble(0, CHART_FIXED_MAX) <= 0 || ChartGetDouble(0, CHART_FIXED_MIN) <= 0) {
    Print("max = ", ChartGetDouble(0, CHART_FIXED_MAX));
    Print("min = ", ChartGetDouble(0, CHART_FIXED_MIN));
    return -1;
  }
  
  int limit = recalcIndex(prev_calculated);
  checkingPatterns(limit);
//--- return value of prev_calculated for next call
  return rates_total;
}